module Main where

import Control.Monad
import Control.Concurrent (threadDelay)
import Data.IORef
import Data.Maybe

import System.Random

import qualified Graphics.UI.Threepenny as UI
import Graphics.UI.Threepenny.Core

import Minesweeper

main :: IO ()
main = do
    startGUI defaultConfig {jsStatic = Just "view"} setup

setup :: Window -> UI ()
setup w = void $ do
    return w # set title "Minesweeper"

    UI.addStyleSheet w "bootstrap.min.css"
    UI.addStyleSheet w "minesweeper.css"

    rng <- liftIO newStdGen
    
    let b = createBoard 30 0.15 rng :: Board
    iob <- liftIO $ newIORef b

    getBody w #+ [
        UI.div #. "container" # set UI.id_ "cont" #+ [
            UI.h1 #+ [string "Minesweeper"],
            UI.h4 #+ [string "Jack Harley jharley@tcd.ie"],
            mkElement "table" # set UI.id_ "grid" #+ rows iob b 0
        ]]

    where
        rows iob b r | r < size b = (mkElement "tr" #+ cells iob b r 0) : rows iob b (r+1)
                     | otherwise = []

        cells iob b r c | c < size b = cell iob b (r,c) : cells iob b r (c+1)
                        | otherwise = []
        
        cell iob b (r,c) = do
            cell <- mkElement "td" #. (squareBgColour b (r,c) ++ " " ++ squareTextColour b (r,c)) 
                                #+ [string $ squareAscii b (r,c)]

            on UI.click cell $ \_ -> do
                liftIO $ modifyIORef' iob $ \oldB -> uncover oldB (r,c)
                b <- liftIO $ readIORef iob
                
                cont <- getElementById w "cont"
                let cont' = fromJust cont
                
                getBody w #+ [
                    UI.div #. "container" # set UI.id_ "cont" # set children [] #+ [
                        UI.h1 #+ [string "Minesweeper"],
                        UI.h4 #+ [string "Jack Harley jharley@tcd.ie"],
                        mkElement "table" # set UI.id_ "grid" #+ rows iob b 0
                    ]]

                delete cont'

            return cell